

var CodeMirror={};
CodeMirror.defaults = {};

var modes = CodeMirror.modes = {}, mimeModes = CodeMirror.mimeModes = {};

CodeMirror.defineMode = function(name, mode) {
  if (!CodeMirror.defaults.mode && name != "null") CodeMirror.defaults.mode = name;
  if (arguments.length > 2) {
    mode.dependencies = [];
    for (var i = 2; i < arguments.length; ++i) mode.dependencies.push(arguments[i]);
  }
  modes[name] = mode;
};

CodeMirror.defineMIME = function(mime, spec) {
  mimeModes[mime] = spec;
};


CodeMirror.resolveMode = function(spec) {
  if (typeof spec == "string" && mimeModes.hasOwnProperty(spec)) {
    spec = mimeModes[spec];
  } else if (spec && typeof spec.name == "string" && mimeModes.hasOwnProperty(spec.name)) {
    var found = mimeModes[spec.name];
    spec = createObj(found, spec);
    spec.name = found.name;
  } else if (typeof spec == "string" && /^[\w\-]+\/[\w\-]+\+xml$/.test(spec)) {
    return CodeMirror.resolveMode("application/xml");
  }
  if (typeof spec == "string") return {name: spec};
  else return spec || {name: "null"};
};

CodeMirror.getMode = function(options, spec) {
  spec = CodeMirror.resolveMode(spec);
  var mfactory = modes[spec.name];
  if (!mfactory) return CodeMirror.getMode(options, "text/plain");
  var modeObj = mfactory(options, spec);
  if (modeExtensions.hasOwnProperty(spec.name)) {
    var exts = modeExtensions[spec.name];
    for (var prop in exts) {
      if (!exts.hasOwnProperty(prop)) continue;
      if (modeObj.hasOwnProperty(prop)) modeObj["_" + prop] = modeObj[prop];
      modeObj[prop] = exts[prop];
    }
  }
  modeObj.name = spec.name;
  return modeObj;
};

CodeMirror.defineMode("null", function() {
  return {token: function(stream) {stream.skipToEnd();}};
});
CodeMirror.defineMIME("text/plain", "null");

var modeExtensions = CodeMirror.modeExtensions = {};
CodeMirror.extendMode = function(mode, properties) {
  var exts = modeExtensions.hasOwnProperty(mode) ? modeExtensions[mode] : (modeExtensions[mode] = {});
  copyObj(properties, exts);
};
  
  
//*******************

function StringStream(string, tabSize) {
  this.pos = this.start = 0;
  this.string = string;
  this.tabSize = tabSize || 8;
  this.lastColumnPos = this.lastColumnValue = 0;
}

StringStream.prototype = {
  eol: function() {return this.pos >= this.string.length;},
  sol: function() {return this.pos == 0;},
  peek: function() {return this.string.charAt(this.pos) || undefined;},
  next: function() {
    if (this.pos < this.string.length)
      return this.string.charAt(this.pos++);
  },
  eat: function(match) {
    var ch = this.string.charAt(this.pos);
    if (typeof match == "string") var ok = ch == match;
    else var ok = ch && (match.test ? match.test(ch) : match(ch));
    if (ok) {++this.pos; return ch;}
  },
  eatWhile: function(match) {
    var start = this.pos;
    while (this.eat(match)){}
    return this.pos > start;
  },
  eatSpace: function() {
    var start = this.pos;
    while (/[\s\u00a0]/.test(this.string.charAt(this.pos))) ++this.pos;
    return this.pos > start;
  },
  skipToEnd: function() {this.pos = this.string.length;},
  skipTo: function(ch) {
    var found = this.string.indexOf(ch, this.pos);
    if (found > -1) {this.pos = found; return true;}
  },
  backUp: function(n) {this.pos -= n;},
  column: function() {
    if (this.lastColumnPos < this.start) {
      this.lastColumnValue = countColumn(this.string, this.start, this.tabSize, this.lastColumnPos, this.lastColumnValue);
      this.lastColumnPos = this.start;
    }
    return this.lastColumnValue;
  },
  indentation: function() {return countColumn(this.string, null, this.tabSize);},
  match: function(pattern, consume, caseInsensitive) {
    if (typeof pattern == "string") {
      var cased = function(str) {return caseInsensitive ? str.toLowerCase() : str;};
      var substr = this.string.substr(this.pos, pattern.length);
      if (cased(substr) == cased(pattern)) {
        if (consume !== false) this.pos += pattern.length;
        return true;
      }
    } else {
      var match = this.string.slice(this.pos).match(pattern);
      if (match && match.index > 0) return null;
      if (match && consume !== false) this.pos += match[0].length;
      return match;
    }
  },
  current: function(){return this.string.slice(this.start, this.pos);}
};

//Pour mode Perl qui surcharge StringStream
CodeMirror.StringStream = StringStream;


function countColumn(string, end, tabSize, startIndex, startValue) {
  if (end == null) {
    end = string.search(/[^\s\u00a0]/);
    if (end == -1) end = string.length;
  }
  for (var i = startIndex || 0, n = startValue || 0; i < end; ++i) {
    if (string.charAt(i) == "\t") n += tabSize - (n % tabSize);
    else ++n;
  }
  return n;
}


function parseLine(pText, pMode, pHandler, pState) {
  if(!pState && pMode.startState) pState = pMode.startState();
  var flattenSpans = pMode.flattenSpans;
  if (flattenSpans == null) flattenSpans = true;
  var curText = "", curStyle = null;
  var stream = new StringStream(pText), style;
  if (pText == "" && pMode.blankLine) pMode.blankLine(pState);
  while (!stream.eol()) {
    style = pMode.token(stream, pState);
    var substr = stream.current();
    stream.start = stream.pos;
    if (!flattenSpans || curStyle != style) {
      if (curText) pHandler.token(curText, curStyle);
      curText = substr; curStyle = style;
    } else curText = curText + substr;
  }
  if (curText) pHandler.token(curText, curStyle);
}


function parseText(pText, pMimeType, pJsonOptions, pHandler, pMaxLines) {
	var vLines = pText.split(/\r\n?|\n/);
	var vMode = CodeMirror.getMode(pJsonOptions ? JSON.parse(pJsonOptions):{}, pMimeType);
	var vState = vMode.startState ? vMode.startState() : null;
	for(var i=0; i < vLines.length && pMaxLines != 0; i++) {
		pHandler.startLine();
		parseLine(vLines[i], vMode, pHandler, vState);
		pHandler.endLine();
		if (pMaxLines>0) pMaxLines--;
	}
}